package com.simplicite.workflows.Demo;

import java.util.*;

import com.simplicite.bpm.*;
import com.simplicite.util.*;
import com.simplicite.util.exceptions.*;
import com.simplicite.util.tools.*;

/**
 * Process DemoOrderCreate
 */
public class DemoOrderCreate extends Processus {
	private static final long serialVersionUID = 1L;
	
	private static final String FIELD = "Field";
	private static final String ROW_ID_FIELD_NAME = "row_id";
	private static final String CREATE_ORDER_ACTIVITY_STEP = "ORDC-300";
	
	Stack<String> orderRowIdStack = new Stack<>();
	
	/**
	 * Méthode appelée par la transition d'activité OrderCreate -> OrderCreate.
	 * Permet d'ajouter à la stack orderRowIdStack le row_id 0 afin de forcer
	 * l'affichage du formulaire de l'objet Commande en création. Cf. postLock.
	 */
	public void addNewOrderHandler(ActivityFile pContext) {
		
		orderRowIdStack.push(ObjectField.DEFAULT_ROW_ID);
		
	}
	
	@Override
	public void postInstantiate(Grant g) {
		
		super.postInstantiate(g);
		
		orderRowIdStack = new Stack<>();
		
	}
	
	@Override
	public void postValidate(ActivityFile context) {
		
		super.postValidate(context);
		
		String step = context.getActivity().getStep();
		
		if (CREATE_ORDER_ACTIVITY_STEP.equals(step)) {
		
			String orderRowId = context.getDataValue(FIELD, ROW_ID_FIELD_NAME);
			
			// On ajoute le row_id de la commande créée à la stack pour
			// permettre à l'utilisateur de revenir en arrière avec le bouton
			// précédent. Cf. postLock.
			orderRowIdStack.push(orderRowId);
			
		}
		
	}
	
	@Override
	public void postLock(ActivityFile context){
		
		String step = context.getActivity().getStep();
		
		if (CREATE_ORDER_ACTIVITY_STEP.equals(step)) {
			
			// Si la stack est vide on affiche le formulaire de création
			// d'une commande (cas où on arrive pour la première fois sur
			// l'activité permettant de créer une commande).
			// Sinon on récupére le row_id de la commande que l'on doit
			// afficher pour que l'utilisateur puisse modifier une commande
			// déjà créé (via utilisation du bouton précédent). 
			// Si l'utilisateur crée des commandes en boucle, le row_id en haut
			// de la stack sera 0 (cf. addNewRowHandler).
			String orderRowId = orderRowIdStack.empty() ? ObjectField.DEFAULT_ROW_ID : orderRowIdStack.pop();
			
			context.setDataFile(FIELD, ROW_ID_FIELD_NAME, orderRowId); 
				
		}
		
	}
	
}
